
/*!
 * EyzyTreeAPI v0.0.37
 * (c) 2019 amsik
 * Released under the MIT License.
 */

var hasChild = function (node) {
    return node.isBatch || node.child.length > 0;
};
var isArray = function (obj) {
    return Array.isArray(obj);
};
function toArray(value) {
    return isArray(value) ? value : [value];
}
//# sourceMappingURL=utils.js.map

var EyzyNode = /** @class */ (function () {
    function EyzyNode(nodes, api, opts) {
        this._api = api;
        this._state = api.state;
        this._tree = api.tree;
        this._props = api.tree.props;
        this._nodes = nodes;
        this._opts = opts;
    }
    Object.defineProperty(EyzyNode.prototype, "length", {
        get: function () {
            var nodes = this._nodes;
            return isArray(nodes)
                ? nodes.length
                : nodes ? 1 : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EyzyNode.prototype, "result", {
        get: function () {
            return this._nodes;
        },
        enumerable: true,
        configurable: true
    });
    EyzyNode.prototype.isSilence = function () {
        return this._opts && this._opts.silence;
    };
    EyzyNode.prototype._operate = function (updateState, operator) {
        var _this = this;
        if (!this._nodes) {
            return false;
        }
        if (this.isSilence()) {
            this._tree.silence = true;
        }
        var result = toArray(this._nodes)
            .map(function (node) { return operator(node, _this._state); })
            .every(function (res) { return false !== res; });
        if (updateState) {
            this._tree.updateState(this._state);
        }
        if (this.isSilence()) {
            this._tree.silence = false;
        }
        return result;
    };
    EyzyNode.prototype.remove = function () {
        var _this = this;
        return this._operate(false, function (node, state) {
            return _this._api.remove(node.id);
        });
    };
    EyzyNode.prototype.empty = function () {
        var _this = this;
        return this._operate(true, function (node, state) {
            if (!hasChild(node)) {
                return false;
            }
            _this._api.core.clearKeys(node, true);
            state.set(node.id, {
                child: [],
                indeterminate: false,
                isBatch: false
            });
            if (node.parent) {
                _this._tree.refreshDefinite(node.id, !!node.checked, false);
            }
        });
    };
    EyzyNode.prototype.select = function (extendSelection, expandOnSelect) {
        var _this = this;
        if (undefined === expandOnSelect) {
            expandOnSelect = this._props.expandOnSelect;
        }
        return this._operate(false, function (node) {
            if (node.selected) {
                return false;
            }
            if (_this._props.multiple) {
                _this._tree.select(node, false, extendSelection);
            }
            else {
                _this._tree.select(node);
            }
            if (expandOnSelect && !node.expanded) {
                _this._tree.expand(node);
            }
        });
    };
    EyzyNode.prototype.unselect = function () {
        var _this = this;
        return this._operate(false, function (node) {
            if (!node.selected) {
                return false;
            }
            _this._tree.unselect(node);
        });
    };
    EyzyNode.prototype.check = function () {
        var _this = this;
        if (!this._props.checkable) {
            return false;
        }
        return this._operate(false, function (node) {
            if (node.checked) {
                return false;
            }
            _this._tree.check(node);
        });
    };
    EyzyNode.prototype.uncheck = function () {
        var _this = this;
        if (!this._props.checkable) {
            return false;
        }
        return this._operate(false, function (node, state) {
            if (!node.checked) {
                return false;
            }
            _this._tree.check(node);
        });
    };
    EyzyNode.prototype.disable = function () {
        return this._operate(true, function (node, state) {
            if (node.disabled) {
                return false;
            }
            state.set(node.id, 'disabled', true);
        });
    };
    EyzyNode.prototype.enable = function () {
        return this._operate(true, function (node, state) {
            if (!node.disabled) {
                return false;
            }
            state.set(node.id, 'disabled', false);
        });
    };
    EyzyNode.prototype.disableCheckbox = function () {
        if (!this._props.checkable) {
            return false;
        }
        return this._operate(true, function (node, state) {
            if (node.disabledCheckbox) {
                return false;
            }
            state.set(node.id, 'disabledCheckbox', true);
        });
    };
    EyzyNode.prototype.enableCheckbox = function () {
        var _this = this;
        if (!this._props.checkable) {
            return false;
        }
        return this._operate(true, function (node, state) {
            if (!node.disabledCheckbox) {
                return false;
            }
            state.set(node.id, 'disabledCheckbox', false);
            if (hasChild(node) && _this._props.useIndeterminateState) {
                var iNode = node.checked
                    ? node
                    : node.child[0];
                _this._tree.refreshDefinite(iNode.id, !!iNode.checked, false);
            }
        });
    };
    EyzyNode.prototype.expand = function (includingDisabled) {
        var _this = this;
        return this._operate(false, function (node) {
            if (!hasChild(node) || node.expanded) {
                return false;
            }
            _this._tree.expand(node, includingDisabled);
        });
    };
    EyzyNode.prototype.collapse = function (includingDisabled) {
        var _this = this;
        return this._operate(false, function (node) {
            if (!hasChild(node) || !node.expanded) {
                return false;
            }
            _this._tree.expand(node, includingDisabled);
        });
    };
    EyzyNode.prototype.data = function (key, value) {
        var _this = this;
        if (!this._nodes) {
            return;
        }
        var nodes = toArray(this._nodes);
        if (1 === nodes.length) {
            return this._api.core.data(nodes[0], key, value);
        }
        return nodes.map(function (node) { return _this._api.core.data(node, key, value); });
    };
    EyzyNode.prototype.hasClass = function (className) {
        var _this = this;
        return toArray(this._nodes).some(function (node) { return _this._api.core.hasClass(node, className); });
    };
    EyzyNode.prototype.addClass = function (classNames) {
        var _this = this;
        toArray(this._nodes).forEach(function (node) { return _this._api.core.addClass(node, classNames); });
        return true;
    };
    EyzyNode.prototype.removeClass = function (classNames) {
        var _this = this;
        return this._operate(false, function (node) {
            if (!node.className) {
                return;
            }
            var oldClassNames = node.className;
            var updatedNode = _this._api.core.removeClass(node, classNames);
            return oldClassNames !== updatedNode.className;
        });
    };
    EyzyNode.prototype.append = function (source, opts) {
        var _this = this;
        return this._operate(false, function (node) {
            return _this._api.core.insert(node, source, opts);
        });
    };
    EyzyNode.prototype.prepend = function (source, opts) {
        var _this = this;
        return this._operate(false, function (node) {
            return _this._api.core.insert(node, source, Object.assign({}, opts, { index: 0 }));
        });
    };
    EyzyNode.prototype.before = function (source) {
        var _this = this;
        return this._operate(false, function (node) {
            return _this._api.core.beside(node, source, 0);
        });
    };
    EyzyNode.prototype.after = function (source) {
        var _this = this;
        return this._operate(false, function (node) {
            return _this._api.core.beside(node, source, 1);
        });
    };
    EyzyNode.prototype._find = function (query, multiple) {
        var core = this._api.core;
        var aNodes = toArray(this._nodes);
        var nodes = core
            .flatMap(aNodes)
            .nodes
            .filter(function (node) { return !~aNodes.indexOf(node); });
        return core.find(nodes, multiple, query);
    };
    EyzyNode.prototype.find = function (query) {
        var node = this._find(query, false);
        return new EyzyNode(node ? [node] : [], this._api, this._opts);
    };
    EyzyNode.prototype.findAll = function (query) {
        var nodes = this._find(query, true);
        return new EyzyNode(nodes && nodes.length ? nodes : [], this._api, this._opts);
    };
    return EyzyNode;
}());

var call = function (api, name, query, multiple, args) {
    return api._operate(query, !!multiple, function (node) {
        return args ? node[name].apply(node, args) : node[name]();
    });
};
var EyzyTreeAPI = /** @class */ (function () {
    function EyzyTreeAPI(api, opts) {
        this._tree = api.tree;
        this._state = api.state;
        this._api = api;
        this.opts = opts || {};
    }
    EyzyTreeAPI.prototype._operate = function (query, multiple, operator) {
        var nodes = multiple
            ? this._api.findAll(query)
            : this._api.find(query);
        if (!nodes || isArray(nodes) && !nodes.length) {
            return false;
        }
        return operator(new EyzyNode(nodes, this._api, this.opts));
    };
    EyzyTreeAPI.prototype.find = function (query) {
        return new EyzyNode(this._api.find(query), this._api, this.opts);
    };
    EyzyTreeAPI.prototype.findAll = function (query) {
        return new EyzyNode(this._api.findAll(query), this._api, this.opts);
    };
    EyzyTreeAPI.prototype.remove = function (query, multiple) {
        return call(this, 'remove', query, multiple);
    };
    EyzyTreeAPI.prototype.empty = function (query, multiple) {
        return call(this, 'empty', query, multiple);
    };
    EyzyTreeAPI.prototype.selected = function () {
        return new EyzyNode(this._api.selected(), this._api, this.opts);
    };
    EyzyTreeAPI.prototype.select = function (query, extendSelection, expandOnSelect) {
        return call(this, 'select', query, false, [extendSelection, expandOnSelect]);
    };
    EyzyTreeAPI.prototype.unselectAll = function () {
        this._api.unselectAll();
    };
    EyzyTreeAPI.prototype.unselect = function (query, multiple) {
        return call(this, 'unselect', query, multiple);
    };
    EyzyTreeAPI.prototype.checked = function (valueConsistsOf, ignoreDisabled) {
        return new EyzyNode(this._api.checked(valueConsistsOf, ignoreDisabled), this._api, this.opts);
    };
    EyzyTreeAPI.prototype.check = function (query, multiple) {
        return call(this, 'check', query, multiple);
    };
    EyzyTreeAPI.prototype.uncheck = function (query, multiple) {
        return call(this, 'uncheck', query, multiple);
    };
    EyzyTreeAPI.prototype.uncheckAll = function () {
        this._api.uncheckAll();
    };
    EyzyTreeAPI.prototype.disable = function (query, multiple) {
        return call(this, 'disable', query, multiple);
    };
    EyzyTreeAPI.prototype.enable = function (query, multiple) {
        return call(this, 'enable', query, multiple);
    };
    EyzyTreeAPI.prototype.disableCheckbox = function (query, multiple) {
        return call(this, 'disableCheckbox', query, multiple);
    };
    EyzyTreeAPI.prototype.enableCheckbox = function (query, multiple) {
        return call(this, 'enableCheckbox', query, multiple);
    };
    EyzyTreeAPI.prototype.expand = function (query, multiple, includingDisabled) {
        return call(this, 'expand', query, multiple, [includingDisabled]);
    };
    EyzyTreeAPI.prototype.collapse = function (query, multiple, includingDisabled) {
        return call(this, 'collapse', query, multiple, [includingDisabled]);
    };
    EyzyTreeAPI.prototype.data = function (query, key, value) {
        return call(this, 'data', query, false, [key, value]);
    };
    EyzyTreeAPI.prototype.hasClass = function (query, className) {
        return call(this, 'hasClass', query, false, [className]);
    };
    EyzyTreeAPI.prototype.addClass = function (query, classNames, multiple) {
        return call(this, 'addClass', query, multiple, [classNames]);
    };
    EyzyTreeAPI.prototype.removeClass = function (query, classNames, multiple) {
        return call(this, 'removeClass', query, multiple, [classNames]);
    };
    EyzyTreeAPI.prototype.append = function (query, source, opts) {
        return call(this, 'append', query, false, [source, opts]);
    };
    EyzyTreeAPI.prototype.prepend = function (query, source, opts) {
        return call(this, 'prepend', query, false, [source, opts]);
    };
    EyzyTreeAPI.prototype.before = function (query, source) {
        return call(this, 'before', query, false, [source]);
    };
    EyzyTreeAPI.prototype.after = function (query, source) {
        return call(this, 'after', query, false, [source]);
    };
    return EyzyTreeAPI;
}());
//# sourceMappingURL=EyzyTreeAPI.js.map

//# sourceMappingURL=index.js.map

export default EyzyTreeAPI;
