
/*!
 * EyzyTree v0.0.37
 * (c) 2019 amsik
 * Released under the MIT License.
 */

(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('react')) :
    typeof define === 'function' && define.amd ? define(['react'], factory) :
    (global.EyzyTree = factory(global.React));
}(this, (function (React) { 'use strict';

    React = React && React.hasOwnProperty('default') ? React['default'] : React;

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */
    /* global Reflect, Promise */

    var extendStatics = function(d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };

    function __extends(d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }

    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    var hasOwnProp = {}.hasOwnProperty;
    function toArray(value) {
        return isArray(value) ? value : [value];
    }
    function callFetcher(node, fn) {
        if (!isCallable(fn)) {
            throw new TypeError('"fetcher" it must be either function or promise');
        }
        var result = isFunction(fn) ? fn(node) : fn;
        if (!result || !result.then) {
            throw new TypeError('"fetcher" property must return a Promise');
        }
        return result;
    }
    function grapObjProps(obj, props) {
        return props.reduce(function (result, name) {
            if (name in obj) {
                result[name] = obj[name];
            }
            return result;
        }, {});
    }
    function isString(obj) {
        return 'string' === typeof obj;
    }
    function isRegExp(obj) {
        return obj instanceof RegExp;
    }
    function isArray(obj) {
        return Array.isArray(obj);
    }
    function isExpandable(node) {
        return !!(node.child && node.child.length) || !!node.isBatch;
    }
    function isCheckable(node) {
        return !(!!node.disabled || !!node.disabledCheckbox);
    }
    function isFunction(value) {
        return 'function' === typeof value;
    }
    function isCallable(value) {
        return isFunction(value) || (value && !!value.then);
    }
    function isEmpty(value) {
        if (isArray(value)) {
            return value.length === 0;
        }
        return !value;
    }
    function get(obj, path) {
        if (!obj) {
            return void 0;
        }
        if (!isArray(path)) {
            path = path.split('.');
        }
        var len = path.length;
        var i = 0;
        while (obj != null && i < len) {
            obj = obj[path[i++]];
        }
        return i && i === len ? obj : undefined;
    }
    function has(targetArray, targetValue) {
        return !!~targetArray.indexOf(targetValue);
    }
    function copyArray(arr) {
        return arr.concat([]);
    }
    function copyObject(obj) {
        var newObj = {};
        for (var i in obj) {
            if (hasOwnProp.call(obj, i)) {
                newObj[i] = isArray(obj[i]) ? copyArray(obj[i]) : obj[i];
            }
        }
        return newObj;
    }
    function remove(targetArr, targetItem) {
        var index = targetArr.indexOf(targetItem);
        if (~index) {
            targetArr.splice(index, 1);
        }
    }
    function isRoot(node) {
        return node && !node.parent;
    }
    function isLeaf(node) {
        return !node.child || (0 === node.child.length && !node.isBatch);
    }
    function isNodeIndeterminate(node, treeCheckedNodes, indeterminateNodes) {
        if (!node.child.length) {
            return false;
        }
        var hasIndeterminate = node.child.some(function (child) {
            return !child.disabled && !child.disabledCheckbox && -1 !== indeterminateNodes.indexOf(child.id);
        });
        if (hasIndeterminate) {
            return true;
        }
        var uncheckedNodes = node.child.reduce(function (count, item) {
            if (true !== item.disabled && true !== item.disabledCheckbox && -1 === treeCheckedNodes.indexOf(item.id)) {
                count++;
            }
            return count;
        }, 0);
        return uncheckedNodes > 0 && uncheckedNodes < node.child.length;
    }

    var hasOwn = {}.hasOwnProperty;
    function cn(args) {
        var result = [];
        if (isArray(args)) {
            return args.join(' ');
        }
        for (var key in args) {
            if (hasOwn.call(args, key) && args[key]) {
                result.push(key);
            }
        }
        return result.join(' ');
    }

    function shallowEqual(objA, objB, keys) {
        if (objA === objB) {
            return true;
        }
        for (var i = 0; i < keys.length; i++) {
            var key = keys[i];
            var valueA = objA[key];
            var valueB = objB[key];
            if (valueA !== valueB) {
                return false;
            }
        }
        return true;
    }

    var hasChild = function (node) {
        return node.isBatch || node.child.length > 0;
    };
    var comparingKeys = [
        'id', 'checked', 'selected', 'child', 'checked', 'expanded', 'hash', 'loading'
    ];
    var Node = /** @class */ (function (_super) {
        __extends(Node, _super);
        function Node() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.handleSelect = function (event) {
                if (_this.props.onSelect) {
                    _this.props.onSelect(_this.getNode(), event);
                }
            };
            _this.handleCheck = function () {
                if (_this.props.onCheck) {
                    _this.props.onCheck(_this.getNode());
                }
            };
            _this.handleExpand = function () {
                if (_this.props.onExpand) {
                    _this.props.onExpand(_this.getNode());
                }
            };
            _this.handleDoubleClick = function (e) {
                if (_this.props.onDoubleClick) {
                    _this.props.onDoubleClick(_this.getNode());
                }
            };
            _this.renderCheckbox = function () {
                if (!_this.props.checkable || _this.props.hidenCheckbox) {
                    return null;
                }
                var Checkbox = _this.props.checkboxRenderer;
                if (!Checkbox) {
                    return React.createElement("span", { className: "node-checkbox", onMouseUp: _this.handleCheck });
                }
                return (React.createElement("span", { className: "node-checkbox-overrided", onMouseUp: _this.handleCheck },
                    React.createElement(Checkbox, { node: _this.getNode() })));
            };
            _this.renderArrow = function () {
                var ArrowRenderer = _this.props.arrowRenderer;
                if (!hasChild(_this.props)) {
                    return React.createElement("span", { className: "node-noop" });
                }
                if (!ArrowRenderer) {
                    return React.createElement("span", { className: "node-arrow", onMouseUp: _this.handleExpand });
                }
                return (React.createElement("span", { className: "node-arrow-extended", onMouseUp: _this.handleExpand },
                    React.createElement(ArrowRenderer, { node: _this.getNode() })));
            };
            return _this;
        }
        Node.prototype.shouldComponentUpdate = function (nextProps) {
            if (0 === nextProps.depth && this.props.hash !== nextProps.hash) {
                return true;
            }
            return !shallowEqual(this.props, nextProps, comparingKeys);
        };
        Node.prototype.getNode = function () {
            return this.props.node;
        };
        Node.prototype.render = function () {
            var _a = this.props, loading = _a.loading, checked = _a.checked, selected = _a.selected, children = _a.children, expanded = _a.expanded, disabled = _a.disabled, disabledCheckbox = _a.disabledCheckbox, className = _a.className, indeterminate = _a.indeterminate, useIndeterminateState = _a.useIndeterminateState, TextRenderer = _a.textRenderer;
            var text = this.props.text;
            var nodeContentClass = cn({
                'node-content': true,
                'has-child': hasChild(this.props),
                'selected': selected,
                'checked': checked,
                'expanded': expanded,
                'disabled': disabled,
                'loading': loading,
                'disabled-checkbox': disabledCheckbox,
                'indeterminate': !checked && indeterminate && false !== useIndeterminateState
            });
            var containerClassNames = ['tree-node'];
            if (className) {
                containerClassNames.push(className);
            }
            return (React.createElement("li", { className: containerClassNames.join(' ') },
                React.createElement("div", { className: nodeContentClass },
                    this.renderArrow(),
                    this.renderCheckbox(),
                    React.createElement("span", { className: "node-text", onMouseUp: this.handleSelect, onDoubleClick: this.handleDoubleClick }, TextRenderer ? React.createElement(TextRenderer, { node: this.getNode() }) : text)),
                children && expanded &&
                    React.createElement("ul", { className: "node-child" }, children)));
        };
        return Node;
    }(React.Component));

    /**
     * Queries:
     *  - api.find(/item \d{2}/) // it will search by text (using RegExp)
     *  - api.find('item') // the same
     *  - api.find(node => node.disabled) // function
     *
     *  - api.find({text: ['Item 1', 'Item 2']}) // KEY OR
     *  - api.find({text: /item/, selected: true}) // AND
     *  - api.find([{text: /item/}, {selected: true}]) // OR
     */
    var specials = {
        disabledCheckbox: function (node, value) { return !value === !node.disabledCheckbox; },
        disabled: function (node, value) { return !value === !node.disabled; },
        expandable: function (node, value) { return value === isExpandable(node); },
        expanded: function (node, value) {
            if (value) {
                return !!node.expanded;
            }
            return isExpandable(node) && !node.expanded;
        },
        selected: function (node, value) { return !value === !node.selected; },
        checked: function (node, value) { return !value === !node.checked; },
        checkable: function (node, value) { return value === isCheckable(node); },
        isLeaf: function (node, value) { return value !== isExpandable(node); },
        $not: function (node, value) {
            var searchQuery = toArray(value).map(parseQuery);
            if ('expanded' in value) {
                searchQuery.push(parseQuery({ expandable: false }));
            }
            return false === matchQuery(node, searchQuery);
        }
    };
    function parseQuery(query) {
        if (isFunction(query)) {
            return query;
        }
        if (isString(query)) {
            return function (node) {
                return testKey(query, node.id) || testKey(query, node.text);
            };
        }
        if (query.id) {
            return function (node) {
                return testKey(query.id, node.id);
            };
        }
        var matches = isRegExp(query)
            ? { text: query }
            : query;
        return function (node) {
            var keys = Object.keys(matches);
            if (!keys.length) {
                return false;
            }
            return keys.every(function (key) {
                if (key in specials) {
                    return toArray(matches[key])
                        .some(function (value) { return testSpecial(specials[key], value, node); });
                }
                return toArray(matches[key])
                    .some(function (value) { return testKey(value, get(node, key)); });
            });
        };
    }
    function testSpecial(s, value, node) {
        return true === s(node, value);
    }
    function testKey(v0, v1) {
        if (isRegExp(v0)) {
            return new RegExp(v0).test(v1);
        }
        return v0 === v1;
    }
    function matchQuery(node, query) {
        return query.some(function (query) { return true === query(node); });
    }
    function find(source, traveler, multiple, query) {
        if (isEmpty(query)) {
            return null;
        }
        var result = [];
        var searchQuery = toArray(query).map(parseQuery);
        var seeker = function (node) {
            if (matchQuery(node, searchQuery)) {
                result.push(node);
                return multiple;
            }
            return true;
        };
        traveler(source, seeker);
        if (!result.length) {
            return multiple ? [] : null;
        }
        return multiple ? result : result[0];
    }

    function recurseDown(obj, fn, excludeSelf, depth) {
        if (depth === void 0) { depth = 0; }
        var res;
        if (Array.isArray(obj)) {
            return obj.map(function (node) { return recurseDown(node, fn, false, depth); });
        }
        if (!excludeSelf) {
            res = fn(obj, depth);
        }
        if (res !== false && obj.child && obj.child.length) {
            res = recurseDown(obj.child, fn, false, depth + 1);
        }
        return res;
    }
    function rootElement(obj) {
        var node = obj.parent;
        while (node) {
            if (!node.parent) {
                return node;
            }
            node = node.parent;
        }
        return null;
    }
    function traverseUp(obj, fn) {
        var node = obj.parent;
        while (node) {
            if (false === fn(node) || !node.parent) {
                return;
            }
            node = node.parent;
        }
    }
    function getFirstChild(node, onlyEnabled) {
        if (onlyEnabled) {
            var enabledNodes = node.child.filter(function (n) { return !n.disabled; });
            return enabledNodes.length ? enabledNodes[0] : null;
        }
        return node.child[0] || null;
    }
    function isRootParentCollapsed(node) {
        var root = rootElement(node);
        return !root ? false : !root.expanded;
    }
    function flatMap(collection, ignoreCollapsed) {
        var result = {
            nodes: [],
            ids: []
        };
        recurseDown(collection, function (node) {
            if (node.disabled) {
                return;
            }
            if (ignoreCollapsed && node.parent && !node.parent.expanded || isRootParentCollapsed(node)) {
                return;
            }
            result.nodes.push(node);
            result.ids.push(node.id);
        });
        return result;
    }
    function walkBreadth(items, cb) {
        var levels = {};
        recurseDown(items, function (item) {
            var depth = item.depth || 0;
            if (!levels[depth]) {
                levels[depth] = [];
            }
            levels[depth].push(item);
        });
        var nodes = Object.keys(levels).reduce(function (nodes, level) {
            nodes.push.apply(nodes, levels[level]);
            return nodes;
        }, []);
        return nodes.some(function (node) { return false === cb(node); });
    }

    var TreeAPI = /** @class */ (function () {
        function TreeAPI(tree, state, core) {
            this.tree = tree;
            this.state = state;
            this.core = core;
        }
        TreeAPI.prototype._operate = function (query, operator) {
            var node = this.find(query);
            if (!node) {
                return null;
            }
            return operator(node);
        };
        TreeAPI.prototype.find = function (query) {
            return find(this.state.get(), walkBreadth, false, query);
        };
        TreeAPI.prototype.findAll = function (query) {
            return find(this.state.get(), walkBreadth, true, query);
        };
        TreeAPI.prototype.selected = function () {
            var state = this.state;
            var selected = this.tree.selected
                .map(function (id) { return state.byId(id); });
            if (true !== this.tree.props.multiple) {
                return selected.length ? selected[0] : null;
            }
            return selected.filter(function (item) { return null !== item; });
        };
        TreeAPI.prototype.checked = function (valueConsistsOf, showDisabled) {
            var state = this.state;
            var checked = [];
            this.tree.checked.forEach(function (id) {
                var node = state.byId(id);
                if (node) {
                    checked.push(node);
                }
            });
            if ('WITH_INDETERMINATE' === valueConsistsOf) {
                this.tree.indeterminate.forEach(function (id) {
                    var node = state.byId(id);
                    if (node) {
                        checked.push(node);
                    }
                });
            }
            if (!showDisabled) {
                checked = checked.filter(isCheckable);
            }
            switch (valueConsistsOf) {
                case 'LEAF': return checked.filter(function (node) { return isLeaf(node); });
                case 'BRANCH':
                    return checked.filter(function (node) {
                        if (node.parent && node.parent.checked) {
                            return false;
                        }
                        return true;
                    });
            }
            return checked;
        };
        TreeAPI.prototype.set = function (query, key, value) {
            var node = this.find(query);
            if (!node) {
                return false;
            }
            // TODO: selected, checked should be dublicated in the tree state
            this.state.set(node.id, key, value);
            this.tree.updateState();
            return true;
        };
        TreeAPI.prototype.data = function (query, key, value) {
            var _this = this;
            return this._operate(query, function (node) {
                return _this.core.data(node, key, value);
            });
        };
        TreeAPI.prototype.addClass = function (query, classNames) {
            var _this = this;
            return this._operate(query, function (node) {
                return _this.core.addClass(node, classNames);
            });
        };
        TreeAPI.prototype.removeClass = function (query, classNames) {
            var _this = this;
            return this._operate(query, function (node) {
                return _this.core.removeClass(node, classNames);
            });
        };
        TreeAPI.prototype.hasClass = function (query, className) {
            var _this = this;
            return this._operate(query, function (node) {
                return _this.core.hasClass(node, className);
            });
        };
        TreeAPI.prototype.append = function (query, source, opts) {
            var _this = this;
            return this._operate(query, function (node) {
                return _this.core.insert(node, source, opts);
            });
        };
        TreeAPI.prototype.prepend = function (query, source, opts) {
            var _this = this;
            return this._operate(query, function (node) {
                return _this.core.insert(node, source, Object.assign({}, opts, { index: 0 }));
            });
        };
        TreeAPI.prototype.before = function (query, source) {
            var _this = this;
            return this._operate(query, function (node) {
                return _this.core.beside(node, source, 0);
            });
        };
        TreeAPI.prototype.after = function (query, source) {
            var _this = this;
            return this._operate(query, function (node) {
                return _this.core.beside(node, source, 1);
            });
        };
        TreeAPI.prototype.remove = function (query) {
            var _this = this;
            return this._operate(query, function (node) {
                return _this.core.remove(node);
            });
        };
        TreeAPI.prototype.uncheckAll = function () {
            return this.core.uncheckAll();
        };
        TreeAPI.prototype.unselectAll = function () {
            return this.core.unselectAll();
        };
        TreeAPI.prototype.toArray = function () {
            return this.state.get();
        };
        return TreeAPI;
    }());

    function iterable(key, value) {
        if ('string' === typeof key) {
            return [[key, value]];
        }
        if (!value) {
            var res = [];
            for (var i in key) {
                if (hasOwnProp.call(key, i)) {
                    res.push([i, key[i]]);
                }
            }
            return res;
        }
        return [];
    }
    function replace(node) {
        if (!node || !node.child) {
            return node;
        }
        node.child = copyArray(node.child);
        node.child.forEach(function (child) {
            var replaced = replace(child);
            replaced.parent = node;
            return replaced;
        });
        return node;
    }
    function itemById(id, targetState) {
        return Object.keys(targetState)
            .find(function (k) { return targetState[k].id === id; }) || null;
    }
    function updateChild(parentNode) {
        parentNode.child.forEach(function (child) {
            child.parent = parentNode;
        });
        return parentNode;
    }
    var State = /** @class */ (function () {
        function State(state) {
            this.nodes = state;
        }
        State.prototype.updateRoot = function (node, iterableValue) {
            var i = itemById(node.id, this.nodes);
            var newObj = copyObject(node);
            if (iterableValue) {
                iterableValue.forEach(function (_a) {
                    var key = _a[0], value = _a[1];
                    node[key] = value;
                    newObj[key] = value;
                });
            }
            if (null !== i) {
                this.nodes[i] = updateChild(newObj);
                return this.nodes[i];
            }
        };
        State.prototype.updateLeaf = function (node, iterableValue) {
            var root = rootElement(node);
            var parentNode = node.parent;
            if (!parentNode || !root || !parentNode.child) {
                return;
            }
            var index = this.getIndex(node);
            if (null === index) {
                return;
            }
            if (iterableValue) {
                iterableValue.forEach(function (_a) {
                    var key = _a[0], value = _a[1];
                    node[key] = value;
                    parentNode.child[index][key] = value;
                });
            }
            this.updateRoot(replace(root));
            return node;
        };
        State.prototype.set = function (id, key, value) {
            var node = this.byId(id);
            if (!node) {
                return;
            }
            if (isRoot(node)) {
                return this.updateRoot(node, iterable(key, value));
            }
            return this.updateLeaf(node, iterable(key, value));
        };
        State.prototype.getIndex = function (node) {
            var parent = node.parent;
            var nodeId = node.id;
            if (parent && parent.child) {
                var childIndex_1 = null;
                parent.child.some(function (node, i) {
                    if (nodeId === node.id) {
                        childIndex_1 = i;
                        return true;
                    }
                    return false;
                });
                return childIndex_1;
            }
            for (var i = 0; i < this.nodes.length; i++) {
                if (this.nodes[i].id === nodeId) {
                    return i;
                }
            }
            return null;
        };
        State.prototype.insertAt = function (parent, nodes, index) {
            var _a;
            if (parent && parent.child) {
                var child = copyArray(parent.child);
                child.splice.apply(child, [index, 0].concat(nodes));
                return child;
            }
            else {
                (_a = this.nodes).splice.apply(_a, [index, 0].concat(nodes));
            }
            return nodes;
        };
        State.prototype.remove = function (id) {
            var node = this.byId(id);
            if (!node) {
                return null;
            }
            var index = this.getIndex(node);
            if (null === index || !~index) {
                return null;
            }
            var parent = node.parent;
            if (!parent) {
                this.nodes.splice(index, 1);
            }
            else {
                var child = copyArray(parent.child);
                child.splice(index, 1);
                this.set(parent.id, 'child', child);
            }
            return node;
        };
        State.prototype.byId = function (id) {
            var node = null;
            recurseDown(this.nodes, function (obj) {
                if (obj.id === id) {
                    node = obj;
                    return false;
                }
            });
            return node;
        };
        State.prototype.get = function () {
            return this.nodes;
        };
        State.prototype.toArray = function () {
            var result = [];
            var state = this.nodes;
            for (var i in state) {
                if (hasOwnProp.call(state, i)) {
                    result.push(state[i]);
                }
            }
            return result;
        };
        return State;
    }());

    function s4() {
        return Math.floor((1 + Math.random()) * 0x10000)
            .toString(16)
            .substring(1);
    }
    function uuid() {
        return s4() + '-' + s4();
    }

    function parseNode(data, parentNode) {
        if (!isArray(data)) {
            return parseNode([data], parentNode);
        }
        var parent = parentNode || null;
        return data
            .reduce(function (result, node) {
            if ('number' === typeof node || !node) {
                return result;
            }
            if ('string' === typeof node) {
                result.push({
                    id: uuid(),
                    text: node,
                    parent: parent,
                    child: [],
                    data: {}
                });
            }
            else if (isArray(node)) {
                result.push.apply(result, parseNode(node, parentNode));
            }
            else {
                node.id = node.id || uuid();
                node.child = isArray(node.child)
                    ? parseNode(node.child, node)
                    : [];
                if (!node.data) {
                    node.data = {};
                }
                node.parent = parent;
                result.push(node);
            }
            return result;
        }, [])
            .filter(function (node) { return node.text && isString(node.text); });
    }

    function parseOpts(opts) {
        try {
            return Object.assign({}, { loading: true }, opts);
        }
        catch (e) {
            return {
                loading: true
            };
        }
    }
    var CoreTree = /** @class */ (function () {
        function CoreTree(tree, state) {
            var _this = this;
            this.flatMap = function (collection, ignoreCollapsed) {
                return flatMap(collection, ignoreCollapsed);
            };
            this.set = function (node, key, value) {
                // TODO: selected, checked should be dublicated in the tree state
                _this.state.set(node.id, key, value);
                _this.tree.updateState();
            };
            this.updateKeys = function (nodes, targetNodes) {
                var tree = _this.tree;
                var state = tree.getState();
                var checked = tree.checked;
                var cascadeCheck = true !== tree.props.noCascade;
                var lastSelected;
                if (targetNodes) {
                    targetNodes.forEach(function (node) {
                        if (node.selected) {
                            lastSelected = node.id;
                        }
                    });
                }
                nodes.forEach(function (parentNode) {
                    var parentDepth = parentNode.depth || 0;
                    recurseDown(parentNode, function (obj, depth) {
                        if (obj.id !== parentNode.id) {
                            obj.depth = parentDepth + depth;
                        }
                        if (cascadeCheck && obj.parent && obj.parent.checked) {
                            obj.checked = true;
                        }
                        if (obj.checked && !has(checked, obj.id)) {
                            checked.push(obj.id);
                        }
                        if (obj.selected) {
                            tree.selected.push(obj.id);
                            if (!lastSelected) {
                                lastSelected = obj.id;
                            }
                        }
                    });
                    if (lastSelected) {
                        tree.selected = tree.selected.filter(function (id) {
                            if (id !== lastSelected) {
                                state.set(id, 'selected', false);
                            }
                            return id === lastSelected;
                        });
                    }
                    if (cascadeCheck) {
                        checked.forEach(function (id) {
                            var node = state.byId(id);
                            if (node && isLeaf(node)) {
                                tree.refreshDefinite(id, true, false);
                            }
                        });
                    }
                });
            };
            this.clearKeys = function (node, includeSelf) {
                if (includeSelf === void 0) { includeSelf = false; }
                var selected = _this.tree.selected;
                var checked = _this.tree.checked;
                var indeterminate = _this.tree.indeterminate;
                recurseDown(node, function (child) {
                    if (child.selected) {
                        remove(selected, child.id);
                    }
                    if (child.checked) {
                        remove(checked, child.id);
                    }
                    remove(indeterminate, child.id);
                }, includeSelf);
            };
            this.load = function (node, resource, showLoading) {
                var result = callFetcher(node, resource);
                if (showLoading) {
                    _this.set(node, 'loading', true);
                }
                return result.then(function (items) {
                    _this.state.set(node.id, {
                        isBatch: false,
                        loading: false
                    });
                    return parseNode(items, node);
                });
            };
            this.beside = function (targetNode, resource, shift) {
                var insertIndex = _this.state.getIndex(targetNode);
                if (null === insertIndex) {
                    return Promise.resolve([]);
                }
                var parent = targetNode.parent;
                var insert = function (nodes) {
                    return _this.insert(parent, nodes, {
                        expand: parent ? parent.expanded : false,
                        loading: false,
                        index: insertIndex + shift
                    });
                };
                if (isCallable(resource)) {
                    return _this.load(targetNode, resource, false).then(function (nodes) {
                        return insert(nodes);
                    });
                }
                else {
                    return insert(resource);
                }
            };
            this.insert = function (parent, resource, opts) {
                opts = parseOpts(opts);
                var tree = _this.tree;
                var state = tree.getState();
                var insert = function (nodes) {
                    var index = undefined !== opts.index
                        ? opts.index
                        : (parent && parent.child ? parent.child.length : 0);
                    var child = state.insertAt(parent, nodes, index);
                    if (parent) {
                        var updatedItem = state.set(parent.id, {
                            child: child
                        });
                        if (updatedItem) {
                            // it must be called before checking 'selectOnExpand'
                            _this.updateKeys([updatedItem], nodes);
                            if (opts.expand && !updatedItem.expanded) {
                                _this.tree.expand(updatedItem);
                            }
                        }
                    }
                    else {
                        _this.updateKeys(nodes);
                    }
                    tree.$emit('Add', parent, nodes);
                    tree.updateState();
                    return nodes
                        .map(function (node) { return state.byId(node.id); })
                        .filter(Boolean);
                };
                if (parent && isCallable(resource)) {
                    return _this.load(parent, resource, opts.loading).then(insert);
                }
                else {
                    return Promise.resolve(insert(parseNode(resource)));
                }
            };
            this.remove = function (node) {
                var tree = _this.tree;
                var id = node.id;
                if (tree.props.checkable && node.checked) {
                    _this.state.set(id, 'checked', false);
                    tree.checked = tree.checked.filter(function (checkedId) { return id !== checkedId; });
                    tree.refreshDefinite(id, false, false);
                }
                var removedNode = _this.state.remove(id);
                if (removedNode) {
                    removedNode.parent = null;
                    _this.clearKeys(removedNode);
                    tree.updateState();
                    tree.$emit('Remove', removedNode);
                }
                return removedNode;
            };
            this.data = function (node, key, value) {
                if (!key && !value) {
                    return node.data;
                }
                if (undefined === value && isString(key)) {
                    return node.data[key];
                }
                var data;
                if (!isString(key)) {
                    data = key;
                }
                else {
                    node.data[key] = value;
                    data = node.data;
                }
                _this.state.set(node.id, 'data', data);
                _this.tree.updateState();
                return node;
            };
            this.hasClass = function (node, className) {
                return !!node.className && new RegExp(className).test(node.className);
            };
            this.uncheckAll = function () {
                var tree = _this.tree;
                if (!tree.props.checkable) {
                    return;
                }
                var state = tree.getState();
                var nodes = _this.find(state.get(), true, [{ checked: true }, { indeterminate: true }]);
                if (nodes) {
                    nodes.forEach(function (node) {
                        _this.tree.$emit('UnCheck', state.set(node.id, {
                            checked: false,
                            indeterminate: false
                        }));
                    });
                }
                tree.updateState();
                tree.checked = [];
                tree.indeterminate = [];
            };
            this.unselectAll = function () {
                var tree = _this.tree;
                var state = tree.getState();
                var nodes = _this.find(state.get(), true, { selected: true });
                if (nodes) {
                    nodes.forEach(function (node) {
                        _this.tree.$emit('UnSelect', state.set(node.id, {
                            selected: false
                        }));
                    });
                }
                tree.updateState();
                tree.selected = [];
            };
            this.state = state;
            this.tree = tree;
        }
        CoreTree.prototype.find = function (target, multiple, query) {
            return find(target, walkBreadth, multiple, query);
        };
        CoreTree.prototype.removeClass = function (node, classNames) {
            var className = (node.className || "")
                .split(' ')
                .filter(function (klazz) { return !has(toArray(classNames), klazz); })
                .join(' ');
            this.set(node, 'className', className);
            return node;
        };
        CoreTree.prototype.addClass = function (node, classNames) {
            var className = node.className ? node.className.split(' ') : [];
            toArray(classNames).forEach(function (klazz) {
                if (!has(className, "" + klazz)) {
                    className.push(klazz);
                }
            });
            this.set(node, 'className', className.join(' '));
            return node;
        };
        return CoreTree;
    }());

    function linkedNode(node, state, ignoreExpanded) {
        var result = {
            current: node
        };
        var ids = flatMap(state.get(), true).ids;
        var currentIndex = ids.indexOf(node.id);
        if (!~currentIndex) {
            return result;
        }
        return {
            current: node,
            parent: node.parent,
            next: state.byId(ids[currentIndex + 1]),
            prev: state.byId(ids[currentIndex - 1])
        };
    }

    var mutatingFields = [
        'checkable',
        'useIndeterminateState',
        'checkboxRenderer',
        'arrowRenderer',
        'textRenderer'
    ];
    var EyzyTree = /** @class */ (function (_super) {
        __extends(EyzyTree, _super);
        function EyzyTree(props) {
            var _this = _super.call(this, props) || this;
            _this.silence = false;
            _this.selected = [];
            _this.checked = [];
            _this.indeterminate = [];
            _this.$emit = function (name, id) {
                var args = [];
                for (var _i = 2; _i < arguments.length; _i++) {
                    args[_i - 2] = arguments[_i];
                }
                if (_this.silence) {
                    return;
                }
                var eventCb = _this.props['on' + name];
                if (!eventCb) {
                    return;
                }
                if (isString(id)) {
                    var node = _this.getState().byId(id);
                    if (node) {
                        eventCb.call.apply(eventCb, [null, node].concat(args));
                    }
                }
                else {
                    eventCb.call.apply(eventCb, [null, id].concat(args));
                }
            };
            _this.refreshDefinite = function (id, willBeChecked, shouldRender) {
                var checked = copyArray(_this.checked);
                var indeterminate = copyArray(_this.indeterminate);
                var state = _this.getState();
                var childIds = [];
                var node = state.byId(id);
                var nodesForEvent = [];
                if (!node) {
                    return;
                }
                recurseDown(node, function (child) {
                    if (!child.disabled && !child.disabledCheckbox) {
                        childIds.push(child.id);
                        if (child.checked !== willBeChecked) {
                            nodesForEvent.push(child.id);
                        }
                    }
                }, true);
                if (willBeChecked) {
                    checked.push.apply(checked, childIds.filter(function (id) { return !has(checked, id); }));
                }
                else {
                    checked = checked.filter(function (nodeId) { return !has(childIds, nodeId); });
                }
                traverseUp(node, function (parentNode) {
                    if (parentNode.disabledCheckbox || parentNode.disabled) {
                        return false;
                    }
                    var isIndeterminate = isNodeIndeterminate(parentNode, checked, indeterminate);
                    var id = parentNode.id;
                    if (isIndeterminate) {
                        if (!has(indeterminate, id)) {
                            indeterminate.push(id);
                        }
                        checked = checked.filter(function (nodeId) { return nodeId !== id; });
                    }
                    else {
                        indeterminate = indeterminate.filter(function (nodeId) { return nodeId !== id; });
                        if (willBeChecked && !has(checked, id)) {
                            checked.push(id);
                        }
                        else {
                            checked = checked.filter(function (nodeId) { return nodeId !== id; });
                        }
                    }
                });
                indeterminate = indeterminate.filter(function (nodeId) { return !has(checked, nodeId); });
                if (willBeChecked) {
                    checked.forEach(function (id) {
                        if (!has(_this.checked, id)) {
                            state.set(id, 'checked', true);
                        }
                    });
                }
                else {
                    _this.checked.forEach(function (id) {
                        if (!has(checked, id)) {
                            state.set(id, 'checked', false);
                        }
                    });
                    childIds.forEach(function (id) {
                        state.set(id, 'checked', false);
                    });
                }
                _this.indeterminate.forEach(function (id) {
                    state.set(id, 'indeterminate', false);
                });
                var useIndeterminateState = false !== _this.props.useIndeterminateState;
                if (useIndeterminateState) {
                    indeterminate.forEach(function (id) { return state.set(id, 'indeterminate', true); });
                }
                _this.checked = checked;
                _this.indeterminate = indeterminate;
                if (false !== shouldRender) {
                    _this.updateState();
                }
                nodesForEvent.forEach(function (id) {
                    _this.$emit('Check', id, willBeChecked);
                });
            };
            _this.getState = function () {
                return _this._state;
            };
            _this.getSelectedNode = function () {
                var lastSelectedNode = _this.selected[_this.selected.length - 1];
                if (!lastSelectedNode) {
                    return null;
                }
                return _this.getState().byId(lastSelectedNode);
            };
            _this.updateState = function (state) {
                if (state === void 0) { state = _this._state; }
                var nodes = state.get();
                if (_this.props.onChange) {
                    _this.props.onChange(state.toArray());
                }
                _this.setState({ nodes: nodes });
            };
            _this.unselect = function (node) {
                if (!node.selected) {
                    return;
                }
                var state = _this.getState();
                state.set(node.id, 'selected', false);
                _this.selected = _this.selected.filter(function (id) {
                    if (id !== node.id) {
                        return true;
                    }
                    if (state.byId(id)) {
                        state.set(id, 'selected', false);
                        _this.$emit('UnSelect', id);
                    }
                    return false;
                });
                _this.updateState();
            };
            _this.select = function (node, ignoreEvent, extendSelection, ignoreUpdating) {
                var state = _this.getState();
                var id = node.id;
                var events = [];
                var multiple = _this.props.multiple;
                if (extendSelection && node.selected) {
                    return _this.unselect(node);
                }
                if (!multiple && node.selected) {
                    return;
                }
                if (!multiple || (multiple && !extendSelection)) {
                    _this.selected = _this.selected.filter(function (nodeId) {
                        var node = state.byId(nodeId);
                        if (node) {
                            state.set(node.id, 'selected', false);
                            events.push(['UnSelect', node.id]);
                        }
                        return false;
                    });
                }
                state.set(id, 'selected', true);
                _this.focused = id;
                _this.selected.push(id);
                if (!ignoreUpdating) {
                    _this.updateState();
                }
                if (true !== ignoreEvent) {
                    events.push(['Select', id]);
                    events.forEach(function (event) { return _this.$emit(event[0], event[1]); });
                }
            };
            _this.selectRange = function (focused, targetNode) {
                var _a = flatMap(_this.state.nodes, true), ids = _a.ids, nodes = _a.nodes;
                var focusedIndex = ids.indexOf(focused);
                var targetIndex = ids.indexOf(targetNode);
                if (!~focusedIndex || !~targetIndex) {
                    return;
                }
                var start = Math.min(focusedIndex, targetIndex);
                var end = Math.max(focusedIndex, targetIndex) + 1;
                var state = _this.getState();
                var willBeSelected = nodes.slice(start, end).map(function (node) { return node.id; });
                var $emits = [];
                _this.selected.forEach(function (id) {
                    if (!has(willBeSelected, id)) {
                        state.set(id, 'selected', false);
                        $emits.push(['UnSelect', id]);
                    }
                });
                _this.selected = willBeSelected.map(function (id) {
                    if (!has(_this.selected, id)) {
                        state.set(id, 'selected', true);
                        $emits.push(['Select', id]);
                    }
                    return id;
                });
                _this.updateState();
                $emits.forEach(function (_a) {
                    var name = _a[0], id = _a[1];
                    _this.$emit(name, id);
                });
            };
            _this.check = function (node) {
                if (!_this.props.checkable || node.disabled || node.disabledCheckbox) {
                    return;
                }
                var state = _this.getState();
                var willBeChecked = !node.checked;
                var id = node.id;
                state.set(id, 'checked', willBeChecked);
                if (willBeChecked) {
                    _this.checked = _this.checked.concat([id]);
                }
                else {
                    _this.checked = _this.checked.filter(function (checkedId) { return id !== checkedId; });
                }
                if (_this.props.selectOnCheck) {
                    _this.select(node);
                }
                if (_this.props.noCascade !== true) {
                    _this.refreshDefinite(node.id, willBeChecked);
                }
                else {
                    _this.updateState();
                }
                _this.$emit('Check', id, willBeChecked);
            };
            _this.expand = function (node, includingDisabled) {
                if (node.disabled && !includingDisabled) {
                    return;
                }
                if (node.isBatch) {
                    return _this.loadChild(node);
                }
                if (!isExpandable(node)) {
                    return;
                }
                var state = _this.getState();
                var selectOnExpand = _this.props.selectOnExpand;
                state.set(node.id, 'expanded', !node.expanded);
                if (selectOnExpand && !node.selected) {
                    _this.select(node);
                }
                _this.updateState();
                _this.$emit('Expand', node.id, !node.expanded);
            };
            _this.handleDoubleClick = function (node) {
                _this.$emit('DoubleClick', node.id);
                if (node.disabled || isLeaf(node) || _this.props.expandOnSelect) {
                    return;
                }
                _this.expand(node);
            };
            _this.handleSelect = function (node, event) {
                if (node.disabled) {
                    return;
                }
                if (_this.props.preventSelectParent && isExpandable(node)) {
                    return _this.expand(node);
                }
                var _a = _this.props, multiple = _a.multiple, checkOnSelect = _a.checkOnSelect, expandOnSelect = _a.expandOnSelect, checkable = _a.checkable;
                if (event.shiftKey && multiple && _this.focused) {
                    return _this.selectRange(_this.focused, node.id);
                }
                _this.select(node, false, event.ctrlKey);
                if (event.ctrlKey) {
                    return;
                }
                if (checkable && checkOnSelect && !node.disabledCheckbox) {
                    _this.check(node);
                }
                else if (expandOnSelect) {
                    _this.expand(node);
                }
            };
            _this.handleKeyUp = function (event) {
                if (false === _this.props.keyboardNavigation) {
                    return;
                }
                var keyCode = event.keyCode;
                var selectedNode = _this.getSelectedNode();
                if (selectedNode) {
                    switch (keyCode) {
                        case 32: // space
                        case 13: // enter
                            if (_this.props.checkable && !selectedNode.disabled && !selectedNode.disabledCheckbox) {
                                _this.check(selectedNode);
                            }
                            else if (!isLeaf(selectedNode)) {
                                _this.expand(selectedNode);
                            }
                            break;
                        case 27: // esc
                            if (_this.props.multiple) {
                                _this.core.unselectAll();
                            }
                            else {
                                _this.unselect(selectedNode);
                            }
                            break;
                        case 39: // right arrow
                            if (!isLeaf(selectedNode)) {
                                if (!selectedNode.expanded) {
                                    _this.expand(selectedNode);
                                }
                                else {
                                    var firstChild = getFirstChild(selectedNode, true);
                                    if (firstChild) {
                                        _this.select(firstChild);
                                    }
                                }
                            }
                            break;
                        case 37: // left arrow 
                            if (isLeaf(selectedNode) || !selectedNode.expanded) {
                                var parentNode = selectedNode.parent;
                                if (parentNode && !parentNode.disabled) {
                                    _this.select(parentNode);
                                }
                            }
                            else if (selectedNode.expanded) {
                                _this.expand(selectedNode);
                            }
                            break;
                        case 40: // bottom arrow
                            var next = linkedNode(selectedNode, _this.getState()).next;
                            if (next) {
                                _this.select(next);
                            }
                            break;
                        case 38: // up arrow
                            var prev = linkedNode(selectedNode, _this.getState()).prev;
                            if (prev) {
                                _this.select(prev);
                            }
                            break;
                    }
                }
            };
            _this.loadChild = function (node) {
                var fetchData = _this.props.fetchData;
                if (!fetchData || node.loading) {
                    return;
                }
                return _this.core.insert(node, fetchData, { expand: true });
            };
            _this.renderNode = function (node) {
                var treePropsKeys = [
                    'checkable', 'arrowRenderer', 'textRenderer', 'checkboxRenderer'
                ];
                var treeProps = treePropsKeys.reduce(function (props, key) {
                    if (_this.props[key]) {
                        props[key] = _this.props[key];
                    }
                    return props;
                }, {});
                treeProps.hash = _this.state.hash;
                treeProps.useIndeterminateState = _this.props.useIndeterminateState;
                return (React.createElement(Node, __assign({ key: node.id, node: node, onSelect: _this.handleSelect, onDoubleClick: _this.handleDoubleClick, onCheck: _this.check, onExpand: _this.expand }, treeProps, node), node.expanded ? node.child.map(_this.renderNode) : null));
            };
            var data = parseNode(props.data || []);
            recurseDown(data, function (obj, depth) {
                obj.depth = depth;
                if (obj.selected) {
                    _this.selected.push(obj.id);
                }
                if (obj.checked) {
                    _this.checked.push(obj.id);
                }
            });
            _this._state = new State(data);
            _this.core = new CoreTree(_this, _this._state);
            if (_this.props.noCascade !== true) {
                _this.checked.forEach(function (id) {
                    var node = _this._state.byId(id);
                    if (!node) {
                        return;
                    }
                    if (isLeaf(node) || !node.parent) {
                        _this.refreshDefinite(id, true, false);
                    }
                });
            }
            _this.state = {
                nodes: _this._state.get(),
                hash: uuid(),
                mutatingFields: grapObjProps(props, mutatingFields)
            };
            return _this;
        }
        EyzyTree.getDerivedStateFromProps = function (nextProps, state) {
            if (!shallowEqual(nextProps, state.mutatingFields, mutatingFields)) {
                return {
                    hash: uuid(),
                    mutatingFields: grapObjProps(nextProps, mutatingFields)
                };
            }
            return null;
        };
        EyzyTree.prototype.componentDidMount = function () {
            if (this.props.onReady) {
                this.props.onReady(new TreeAPI(this, this._state, this.core));
            }
        };
        EyzyTree.prototype.render = function () {
            var _this = this;
            var props = this.props;
            var treeClass = 'theme' in props
                ? 'eyzy-tree ' + props.theme
                : 'eyzy-tree eyzy-theme';
            return (React.createElement("ul", { className: treeClass, tabIndex: -1, onKeyDown: this.handleKeyUp }, this.state.nodes.map(function (node) {
                return _this.renderNode(node);
            })));
        };
        EyzyTree.TreeNode = Node;
        return EyzyTree;
    }(React.Component));

    return EyzyTree;

})));
//# sourceMappingURL=eyzy-tree.js.map
